@@ -1,6 +1,6 @@
 /*  dctrl-tools - Debian control file inspection tools
     Copyright © 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
-                2010, 2011
+                2010, 2011, 2012, 2013, 2015
                 Antti-Juhani Kaijanaho
 
     This program is free software; you can redistribute it and/or modify
@@ -33,11 +33,12 @@
 #include "fnutil.h"
 #include "fsaf.h"
 #include "i18n.h"
+#include "ifile.h"
 #include "misc.h"
 #include "msg.h"
 #include "paragraph.h"
 #include "predicate.h"
-#include "rc.h"
+#include "strlist.h"
 #include "util.h"
 
 const char * argp_program_version = "grep-dctrl (dctrl-tools) " VERSION;
@@ -51,8 +52,7 @@ static char progdoc [] = N_("grep-dctrl -- grep Debian control files");
 static char argsdoc [] = N_("FILTER [FILENAME...]");
 
 enum {
-        OPT_CONFIG=256,
-        OPT_OPTPARSE,
+        OPT_OPTPARSE=256,
         OPT_SILENT,
         OPT_EQ,
         OPT_LT,
@@ -63,90 +63,46 @@ enum {
         OPT_IGN_ERRS,
         OPT_ENSURE,
         OPT_COMPAT,
-        OPT_PATTERN
+        OPT_PATTERN,
+        OPT_ERRORLEVEL,
 };
 
-#undef BANNER
-
-#ifdef BANNER
-void banner(bool automatic)
-{
-	char * fname = fnqualify_xalloc("~/.grep-dctrl-banner-shown");
-	struct stat st;
-	if (automatic) {
-		int r = stat(fname, &st);
-		if (r == 0) goto end;
-	}
-	FILE * fp = fopen("/dev/tty", "w");
-	if (fp == 0) {
-		perror("/dev/tty");
-		goto end;
-	}
-	fprintf(fp,
-		"==========================================================================\n"
-		"                                  NOTE                                    \n"
-		" grep-dctrl has been rewritten from scratch.  Although this does add new  \n"
-		" features, regressions are certainly possible.  Please watch for them and \n"
-		" report them to the BTS.                                                  \n"
-		"==========================================================================\n"
-		"(The above annoying banner will not be shown to you again, unless you\n"
-		"request it with the -B switch.  It will also be removed entirely soon.)\n");
-
-	int r = creat(fname, 0644);
-	if (r == -1) perror(fname);
-
-	if (!automatic) exit(0);
-
-	for (int i = 15; i > 0; i--) {
-		fprintf(fp, "%2d seconds until program is resumed...\r", i);
-		fflush(fp);
-		sleep(1);
-	}
-	fprintf(fp, "                                       \r");
-	fflush(fp);
-end:
-	free(fname);
-}
-#endif
-
 static struct argp_option options[] = {
-#ifdef BANNER
-	{ "banner",	    'B', 0,		    0, N_("Show the testing banner.") },
-#endif
-	{ "errorlevel",	    'l', N_("LEVEL"),	    0, N_("Set log level to LEVEL.") },
-	{ "field",	    'F', N_("FIELD,FIELD,..."), 0, N_("Restrict pattern matching to the FIELDs given.") },
-	{ 0,		    'P', 0,		    0, N_("This is a shorthand for -FPackage.") },
-	{ 0,		    'S', 0,		    0, N_("This is a shorthand for -FSource:Package.") },
-	{ "show-field",	    's', N_("FIELD,FIELD,..."), 0, N_("Show only the body of these fields from the matching paragraphs.") },
-	{ 0,		    'd', 0,		    0, N_("Show only the first line of the \"Description\" field from the matching paragraphs.") },
-	{ "no-field-names", 'n', 0,		    0, N_("Suppress field names when showing specified fields.") },
-	{ "eregex",	    'e', 0,		    0, N_("Regard the pattern as an extended POSIX regular expression.") },
-	{ "regex",	    'r', 0,		    0, N_("Regard the pattern as a standard POSIX regular expression.") },
-	{ "ignore-case",    'i', 0,		    0, N_("Ignore case when looking for a match.") },
-	{ "invert-match",   'v', 0,		    0, N_("Show only paragraphs that do not match.") },
-        { "invert-show",    'I', 0,                 0, N_("Show those fields that have NOT been selected with -s") },
-	{ "count",	    'c', 0,		    0, N_("Show only the count of matching paragraphs.") },
-	{ "config-file",    OPT_CONFIG, N_("FNAME"),0, N_("Use FNAME as the config file.") },
-	{ "exact-match",    'X', 0,		    0, N_("Do an exact match.") },
-	{ "copying",	    'C', 0,		    0, N_("Print out the copyright license.") },
-	{ "and",	    'a', 0,		    0, N_("Conjunct filters.") },
-	{ "or",		    'o', 0,		    0, N_("Disjunct filters.") },
-	{ "not",	    '!', 0,		    0, N_("Negate the following filters.") },
-	{ "eq",		    OPT_EQ, 0,		    0, N_("Test for version number equality.") },
-	{ "lt",		    OPT_LT, 0,		    0, N_("Version number comparison: <<.") },
-	{ "le",		    OPT_LE, 0,		    0, N_("Version number comparison: <=.") },
-	{ "gt",		    OPT_GT, 0,		    0, N_("Version number comparison: >>.") },
-	{ "ge",		    OPT_GE, 0,		    0, N_("Version number comparison: >=.") },
-	{ "debug-optparse", OPT_OPTPARSE, 0,	    0, N_("Debug option parsing.") },
-	{ "quiet",	    'q', 0,		    0, N_("Do not output to stdout.") },
-	{ "silent",	    OPT_SILENT, 0,	    0, N_("Do not output to stdout.") },
-	{ "mmap",           OPT_MMAP, 0,            0, N_("Attempt mmapping input files") },
-	{ "ignore-parse-errors", OPT_IGN_ERRS, 0,   0, N_("Ignore parse errors") },
-        { "pattern",        OPT_PATTERN, N_("PATTERN"), 0, N_("Specify the pattern to search for") },
-	{ "whole-pkg",	    'w', 0,                 0, N_("Match only whole package names (this implies -e)") },
-        { "ensure-dctrl",   OPT_ENSURE, 0,          0, N_("Ensure that the output is in dctrl format (overridden by -n)") },
-        { "compat",         OPT_COMPAT, 0,          0, N_("Override the effect of an earlier --ensure-dctrl") },
-	{ 0 }
+	{ "errorlevel",	    OPT_ERRORLEVEL, N_("LEVEL"), 0, N_("Set log level to LEVEL."), 0 },
+	{ "field",	    'F', N_("FIELD,FIELD,..."), 0, N_("Restrict pattern matching to the FIELDs given."), 0 },
+	{ 0,		    'P', 0,		    0, N_("This is a shorthand for -FPackage."), 0 },
+	{ 0,		    'S', 0,		    0, N_("This is a shorthand for -FSource:Package."), 0 },
+	{ "show-field",	    's', N_("FIELD,FIELD,..."), 0, N_("Show only the body of these fields from the matching paragraphs."), 0 },
+	{ 0,		    'd', 0,		    0, N_("Show only the first line of the \"Description\" field from the matching paragraphs."), 0 },
+	{ "no-field-names", 'n', 0,		    0, N_("Suppress field names when showing specified fields."), 0 },
+        { "files-with-matches", 'l', 0,             0, N_("Show only the names of the files that contain matching paragraphs."), 0 },
+        { "files-without-matches", 'L', 0,          0, N_("Show only the names of the files that do not contain matching paragraphs."), 0 },
+	{ "eregex",	    'e', 0,		    0, N_("Regard the pattern as an extended POSIX regular expression."), 0 },
+	{ "regex",	    'r', 0,		    0, N_("Regard the pattern as a standard POSIX regular expression."), 0 },
+	{ "ignore-case",    'i', 0,		    0, N_("Ignore case when looking for a match."), 0 },
+	{ "invert-match",   'v', 0,		    0, N_("Show only paragraphs that do not match."), 0 },
+        { "invert-show",    'I', 0,                 0, N_("Show those fields that have NOT been selected with -s"), 0 },
+	{ "count",	    'c', 0,		    0, N_("Show only the count of matching paragraphs."), 0 },
+	{ "exact-match",    'X', 0,		    0, N_("Do an exact match."), 0 },
+	{ "copying",	    'C', 0,		    0, N_("Print out the copyright license."), 0 },
+	{ "and",	    'a', 0,		    0, N_("Conjunct filters."), 0 },
+	{ "or",		    'o', 0,		    0, N_("Disjunct filters."), 0 },
+	{ "not",	    '!', 0,		    0, N_("Negate the following filters."), 0 },
+	{ "eq",		    OPT_EQ, 0,		    0, N_("Test for version number equality."), 0 },
+	{ "lt",		    OPT_LT, 0,		    0, N_("Version number comparison: <<."), 0 },
+	{ "le",		    OPT_LE, 0,		    0, N_("Version number comparison: <=."), 0 },
+	{ "gt",		    OPT_GT, 0,		    0, N_("Version number comparison: >>."), 0 },
+	{ "ge",		    OPT_GE, 0,		    0, N_("Version number comparison: >=."), 0 },
+	{ "debug-optparse", OPT_OPTPARSE, 0,	    0, N_("Debug option parsing."), 0 },
+	{ "quiet",	    'q', 0,		    0, N_("Do not output to stdout."), 0 },
+	{ "silent",	    OPT_SILENT, 0,	    0, N_("Do not output to stdout."), 0 },
+	{ "mmap",           OPT_MMAP, 0,            0, N_("Attempt mmapping input files"), 0 },
+	{ "ignore-parse-errors", OPT_IGN_ERRS, 0,   0, N_("Ignore parse errors"), 0 },
+        { "pattern",        OPT_PATTERN, N_("PATTERN"), 0, N_("Specify the pattern to search for"), 0 },
+	{ "whole-pkg",	    'w', 0,                 0, N_("Match only whole package names (this implies -e)"), 0 },
+        { "ensure-dctrl",   OPT_ENSURE, 0,          0, N_("Ensure that the output is in dctrl format (overridden by -n)"), 0 },
+        { "compat",         OPT_COMPAT, 0,          0, N_("Override the effect of an earlier --ensure-dctrl"), 0 },
+	{ 0, 0, 0, 0, 0, 0 }
 };
 
 
@@ -185,8 +141,6 @@ struct arguments {
 	size_t num_show_fields;
 	/* A machine-readable representation of the predicate.  */
 	struct predicate * p;
-	/* Configuration file name */
-	char const * rcname;
 	/* Ignore parse errors? */
 	bool ignore_errors;
 	/* Quiet operation? */
@@ -206,6 +160,8 @@ struct arguments {
 	bool invert_match;
         /* Show fields that are NOT listed? */
         bool invert_show;
+        /* Show only file name? */
+        bool only_file_name;
 	/* First unused position in toks.  */
 	size_t toks_np;
         /* Token read position. */
@@ -220,13 +176,6 @@ struct arguments {
         size_t show_fields[MAX_FIELDS];
 };
 
-#define IS_SHOW_FIELD(field_app_data) ((field_app_data) & 1)
-#define SET_SHOW_FIELD(field_app_data,val) \
-  ((field_app_data) = ((field_app_data & ~1) | val))
-#define GET_BACKUP_FIELD(field_app_data) (((field_app_data & ~0)) == (unsigned)-1 ? (size_t)-1 : (size_t)(field_app_data) >> 1)
-#define SET_BACKUP_FIELD(field_app_data,val) \
-  ((field_app_data) = (((field_app_data)&1) | (val<<1)))
-
 #define APPTOK(tok) do { apptok(args, (tok)); } while (0)
 
 static void apptok(struct arguments * args, const int tok)
@@ -257,10 +206,6 @@ static error_t parse_opt (int key, char * arg, struct argp_state * state)
 	case 'C':
 		if (!to_stdout (COPYING)) fail();
 		exit(0);
-#ifdef BANNER
-	case 'B':
-		banner(false);
-#endif
         case OPT_ENSURE:
                 args->ensure_dctrl = true;
                 break;
@@ -268,6 +213,13 @@ static error_t parse_opt (int key, char * arg, struct argp_state * state)
                 args->ensure_dctrl = false;
                 break;
 	case 'v':
+                if (args->only_file_name) {
+                        message(L_FATAL, 0,
+                                args->invert_match
+                                ? _("-v is incompatible with -L")
+                                : _("-v is incompatible with -l"));
+                        fail();
+                }
 		args->invert_match = true;
 		break;
 	case 'c':
@@ -281,6 +233,31 @@ static error_t parse_opt (int key, char * arg, struct argp_state * state)
 		debug_message("parse_opt: n", 0);
 		args->show_field_name = false;
 		break;
+	case 'l':
+                if (args->only_file_name) {
+                        message(L_FATAL, 0,
+                                _("only one -l or -L is allowed"));
+                        fail();
+                }
+                if (args->invert_match) {
+                        message(L_FATAL, 0, _("-l is incompatible with -v"));
+                        fail();
+                }
+		args->only_file_name = true;
+		break;
+	case 'L':
+                if (args->only_file_name) {
+                        message(L_FATAL, 0,
+                                _("only one -l or -L is allowed"));
+                        fail();
+                }
+                if (args->invert_match) {
+                        message(L_FATAL, 0, _("-L is incompatible with -v"));
+                        fail();
+                }
+		args->only_file_name = true;
+                args->invert_match = true;
+		break;
 	case 'd':
 		args->short_descr = true;
 		break;
@@ -305,20 +282,20 @@ static error_t parse_opt (int key, char * arg, struct argp_state * state)
                         if (fa == description_attr) {
                                 args->description_selected = true;
                         }
-                        SET_SHOW_FIELD(fa->application_data, true);
+                        fa->is_show_field = true;
 
                         size_t repl_inx = repl == NULL
                                 ? (size_t)(-1)
                                 : fieldtrie_insert(repl)->inx;
 
-                        SET_BACKUP_FIELD(fa->application_data, repl_inx);
+                        fa->backup_field = repl_inx;
 
 			++args->num_show_fields;
 		}
 		free(carg);
 	}
 		break;
-	case 'l': {
+	case OPT_ERRORLEVEL: {
 		int ll = str2loglevel(arg);
 		if (ll < 0)
 		{
@@ -395,7 +372,7 @@ static error_t parse_opt (int key, char * arg, struct argp_state * state)
 		break;
 	case OPT_MMAP:
 		debug_message("parse_opt: mmap", 0);
-		fsaf_mmap = 1;
+                /* ignore */
 		break;
 	case 'i':
 		debug_message("parse_opt: i", 0);
@@ -447,10 +424,6 @@ static error_t parse_opt (int key, char * arg, struct argp_state * state)
 	case ARGP_KEY_ERROR: case ARGP_KEY_FINI: case ARGP_KEY_NO_ARGS:
 		debug_message("parse_opt: ignored", 0);
 		break;
-	case OPT_CONFIG:
-		debug_message("parse_opt: --config-file", 0);
-		args->rcname = strdup(arg);
-		break;
 	default:
 		return ARGP_ERR_UNKNOWN;
 	}
@@ -558,88 +531,109 @@ static void unexpected(int tok)
 	}
 }
 
-static struct predicate * parse_conj(struct arguments * args);
+struct predicate_qualifiers {
+        struct strlist *fields;
+        enum matching_mode mm;
+        bool ignore_case;
+        bool whole_pkg;
+};
 
-static struct predicate * parse_prim(struct arguments * args)
+static struct predicate * parse_conj(struct arguments * args,
+                                     struct predicate_qualifiers pq);
+
+/* prim -> primtok
+   prim -> primtok prim
+   prim -> TOK_LP conj TOK_RP
+   prim -> TOK_PAT prim'
+   prim -> TOK_STR prim'
+
+   prim' ->
+   prim' -> primtok prim'
+
+   primtok -> TOK_FIELD
+   primtok -> TOK_ERGEX | TOK_REGEX
+   primtok -> TOK_ICASE | TOK_EXACT | TOK_WHOLE
+   primtok -> TOK_EQ | TOK_LT | TOK_LE | TOK_GE | TOK_GT
+*/
+static struct predicate * parse_prim(struct arguments * args,
+                                     struct predicate_qualifiers pq_)
 {
-	if (peek_token(args) == TOK_LP) {
-		get_token(args);
-		struct predicate * rv = parse_conj(args);
-		if (get_token(args) != TOK_RP) {
-			message(L_FATAL, 0, _("missing ')' in command line"));
-			fail();
-		}
-		return rv;
-	}
-
         char *pattern = 0;
-        char *fields[MAX_FIELDS];
-        size_t num_fields = 0;
-        enum matching_mode mm = M_SUBSTR;
-        bool ignore_case = false;
-        bool whole_pkg = false;
-        bool nonempty = false;
+        struct predicate *rv = 0;
 
+        struct predicate_qualifiers pq = pq_;
+        struct strlist_memento mem = strlist_save(pq.fields);
+
+        bool nonempty = false;
         while (1) {
-                debug("tok = %s, mm = %d", tokdescr(peek_token(args)), mm);
+                debug("tok = %s, mm = %d", tokdescr(peek_token(args)), pq.mm);
                 switch (peek_token(args)) {
                 case TOK_FIELD:
-                        if (num_fields >= MAX_FIELDS) {
-                                message(L_FATAL, 0, _("too many field names"));
-                                fail();
+                        if (!strlist_append(pq.fields, get_string(args))) {
+                                enomem(0);
                         }
-                        fields[num_fields++] = get_string(args);
                         break;
                 case TOK_ERGEX:
-                        if (mm != M_SUBSTR) goto failmode;
-                        mm = M_EREGEX;
+                        if (pq.mm != M_SUBSTR) goto failmode;
+                        pq.mm = M_EREGEX;
                         get_token(args);
                         break;
                 case TOK_REGEX:
-                        if (mm != M_SUBSTR) goto failmode;
-                        mm = M_REGEX;
+                        if (pq.mm != M_SUBSTR) goto failmode;
+                        pq.mm = M_REGEX;
                         get_token(args);
                         break;
                 case TOK_ICASE:
-                        ignore_case = true;
+                        pq.ignore_case = true;
                         get_token(args);
                         break;
                 case TOK_EXACT:
-                        if (mm != M_SUBSTR) goto failmode;
-                        mm = M_EXACT;
+                        if (pq.mm != M_SUBSTR) goto failmode;
+                        pq.mm = M_EXACT;
                         get_token(args);
                         break;
                 case TOK_WHOLE:
-                        if (mm != M_SUBSTR) goto failmode;
-                        mm = M_EREGEX;
-                        whole_pkg = true;
+                        if (pq.mm != M_SUBSTR) goto failmode;
+                        pq.mm = M_EREGEX;
+                        pq.whole_pkg = true;
                         get_token(args);
                         break;
                 case TOK_EQ:
-                        if (mm != M_SUBSTR) goto failmode;
-                        mm = M_VER_EQ;
+                        if (pq.mm != M_SUBSTR) goto failmode;
+                        pq.mm = M_VER_EQ;
                         get_token(args);
                         break;
                 case TOK_LT:
-                        if (mm != M_SUBSTR) goto failmode;
-                        mm = M_VER_LT;
+                        if (pq.mm != M_SUBSTR) goto failmode;
+                        pq.mm = M_VER_LT;
                         get_token(args);
                         break;
                 case TOK_LE:
-                        if (mm != M_SUBSTR) goto failmode;
-                        mm = M_VER_LE;
+                        if (pq.mm != M_SUBSTR) goto failmode;
+                        pq.mm = M_VER_LE;
                         get_token(args);
                         break;
                 case TOK_GT:
-                        if (mm != M_SUBSTR) goto failmode;
-                        mm = M_VER_GT;
+                        if (pq.mm != M_SUBSTR) goto failmode;
+                        pq.mm = M_VER_GT;
                         get_token(args);
                         break;
                 case TOK_GE:
-                        if (mm != M_SUBSTR) goto failmode;
-                        mm = M_VER_GE;
+                        if (pq.mm != M_SUBSTR) goto failmode;
+                        pq.mm = M_VER_GE;
                         get_token(args);
                         break;
+                case TOK_LP:
+                {
+                        get_token(args);
+                        rv = parse_conj(args, pq);
+                        if (get_token(args) != TOK_RP) {
+                                message(L_FATAL, 0,
+                                        _("missing ')' in command line"));
+                                fail();
+                        }
+                        goto finally;
+                }
                 case TOK_PAT:
                         if (pattern != 0) {
                                 message(L_FATAL, 0,
@@ -655,84 +649,113 @@ static struct predicate * parse_prim(struct arguments * args)
                 default:
                         goto loop_done;
                 }
-                 nonempty = true;
-         } loop_done:
+                nonempty = true;
+        } loop_done:
 
-         if (!nonempty) {
-                 unexpected(get_token(args));
+        if (!nonempty) {
+                unexpected(get_token(args));
+        }
+        
+        if (pattern == 0) {
+                message(L_FATAL, 0, _("A pattern is mandatory"));
+                fail();
+        }
+        
+        if (strlist_is_empty(pq.fields)) {
+                strlist_append(pq.fields, 0);
          }
-
-         if (pattern == 0) {
-                 message(L_FATAL, 0, _("A pattern is mandatory"));
-                 fail();
-         }
-
-         if (num_fields == 0) {
-                 num_fields = 1;
-                 fields[0] = 0;
-         }
-         
-         struct predicate *rv = 0;
-         for (size_t i = 0; i < num_fields; i++) {
-                 struct atom * atom = malloc(sizeof *atom);
-                 if (atom == 0) enomem(0);
-                 atom->field_name = fields[i];
-                 atom->field_inx = -1;
-                 atom->mode = mm;
-                 atom->ignore_case = ignore_case;
-                 atom->whole_pkg = whole_pkg;
-                 atom->pat = pattern;
-                 atom->patlen = strlen(pattern);
-                 atom_finish(atom);
-                 struct predicate *tmp = predicate_ATOM(atom);
-                 rv = rv != 0 ? predicate_OR(rv, tmp) : tmp;
-         }
-
+        
+        for (struct strlist_iterator it = strlist_begin(pq.fields);
+             !strlist_iterator_at_end(it); strlist_iterator_next(&it)) {
+                struct atom * atom = malloc(sizeof *atom);
+                if (atom == 0) enomem(0);
+                atom->field_name = strlist_iterator_get(it);
+                atom->field_inx = -1;
+                atom->mode = pq.mm;
+                atom->ignore_case = pq.ignore_case;
+                atom->whole_pkg = pq.whole_pkg;
+                atom->pat = pattern;
+                atom->patlen = strlen(pattern);
+                atom_finish(atom);
+                struct predicate *tmp = predicate_ATOM(atom);
+                rv = rv != 0 ? predicate_OR(rv, tmp) : tmp;
+        }
+        
+finally:
+        strlist_restore(mem);
         return rv;
 failmode:
         message(L_FATAL, 0, _("inconsistent modifiers of simple filters")); 
         fail();
-        return 0;
+        rv = 0;
+        goto finally;
 }
 
-static struct predicate * parse_neg(struct arguments * args)
+/* neg -> TOK_NOT prim
+   neg -> prim
+*/
+static struct predicate * parse_neg(struct arguments * args,
+                                    struct predicate_qualifiers pq)
 {
 	bool neg = false;
 	if (peek_token(args) == TOK_NOT) {
 		neg = true;
 		get_token(args);
 	}
-	struct predicate * rv = parse_prim(args);
+	struct predicate * rv = parse_prim(args, pq);
 	if (neg) rv = predicate_NOT(rv);
         return rv;
 }
 
-static struct predicate * parse_disj(struct arguments * args)
+/* disj -> neg
+   disj -> disj TOK_OR neg
+*/
+static struct predicate * parse_disj(struct arguments * args,
+                                     struct predicate_qualifiers pq)
 {
-	struct predicate * rv = parse_neg(args);
+	struct predicate * rv = parse_neg(args, pq);
 	while (peek_token(args) == TOK_OR) {
 		get_token(args);
-		struct predicate * tmp = parse_neg(args);
+		struct predicate * tmp = parse_neg(args, pq);
                 rv = predicate_OR(rv, tmp);
 	}
         return rv;
 }
 
-static struct predicate * parse_conj(struct arguments * args)
+/* conj -> disj
+   conj -> conj TOK_AND disj
+*/
+static struct predicate * parse_conj(struct arguments * args,
+                                     struct predicate_qualifiers pq)
 {
-	struct predicate * rv = parse_disj(args);
+	struct predicate * rv = parse_disj(args, pq);
 	while (peek_token(args) == TOK_AND) {
 		get_token(args);
-		struct predicate * tmp = parse_disj(args);
+		struct predicate * tmp = parse_disj(args, pq);
                 rv = predicate_AND(rv, tmp);
 	}
         return rv;
 }
 
+/* predicate -> conj files
+   files ->
+   files -> TOK_STR files
+*/
 static void parse_predicate(struct arguments * args)
 {
 	args->toks_pos = 0;
-	args->p = parse_conj(args);
+
+        struct predicate_qualifiers pq;
+        pq.fields = strlist_new();
+        if (pq.fields == 0) enomem(0);
+        pq.mm = M_SUBSTR;
+        pq.ignore_case = false;
+        pq.whole_pkg = false;
+
+	args->p = parse_conj(args, pq);
+
+        strlist_free(pq.fields);
+
         while (peek_token(args) == TOK_STR) {
                 if (args->num_fnames >= MAX_FNAMES) {
                         message(L_FATAL, 0, _("too many file names"));
@@ -756,7 +779,7 @@ static void show_field(struct arguments *args,
         struct field_data fds =
                 find_field_wr(para,
                               fa->inx,
-                              GET_BACKUP_FIELD(fa->application_data));
+                              fa->backup_field);
         for (struct field_datum *fd = fds.first; fd != NULL; fd = fd->next) {
                 struct fsaf_read_rv r =
                         fsaf_read(para->common->fp,
@@ -787,6 +810,40 @@ static void show_field(struct arguments *args,
         }
 }
 
+static void print_para(struct arguments *args,
+                       struct paragraph *para)
+{
+        if (args->num_show_fields == 0) {
+                struct fsaf_read_rv r = get_whole_para(para);
+                fwrite(r.b, 1, r.len, stdout);
+                putchar('\n');
+                putchar('\n');
+                return;
+        }
+        if (args->invert_show) {
+                for (size_t j = 0;
+                     j < fieldtrie_count() && j < para->nfields; 
+                     j++) {
+                        struct field_attr *fa = fieldtrie_get(j);
+                        if (fa->is_show_field) continue;
+                        show_field(args, para, fa);
+                }
+                puts("");
+        } else {
+                for (size_t j = 0; j < args->num_show_fields; j++) {
+                        size_t inx = args->show_fields[j];
+                        struct field_attr *fa = fieldtrie_get(inx);
+                        assert(fa->is_show_field);
+                        show_field(args, para, fa);
+                }
+                if ((args->show_field_name && args->ensure_dctrl) ||
+                    args->num_show_fields > 1) {
+                        puts("");
+                }
+        }
+}
+
+
 static struct argp argp = { .options = options,
 			    .parser = parse_opt,
 			    .args_doc = argsdoc,
@@ -805,13 +862,10 @@ int main (int argc, char * argv[])
 	msg_set_progname(argv[0]);
 	description_attr = fieldtrie_insert(description);
 	argp_parse (&argp, argc, argv, ARGP_IN_ORDER, 0, &args);
-#ifdef BANNER
-	banner(true);
-#endif
         if (debug_optparse) {
                 fflush(stderr);
                 fputs("tokens:", stdout);
-                for (int i = 0; i < args.toks_np; i++) {
+                for (size_t i = 0; i < args.toks_np; i++) {
                         putchar(' ');
                         fputs(tokdescr(args.toks[i]), stdout);
                         if (args.strings[i] != 0) {
@@ -838,7 +892,7 @@ int main (int argc, char * argv[])
 		}
 		message(L_INFORMATIONAL, 0,
 			_("Adding \"Description\" to selected output fields because of -d"));
-                SET_SHOW_FIELD(description_attr->application_data, 1);
+                description_attr->is_show_field = 1;
                 args.show_fields[args.num_show_fields] = description_attr->inx;
 		++args.num_show_fields;
 	}
@@ -861,14 +915,31 @@ int main (int argc, char * argv[])
 		int fd;
 		struct ifile fname;
 		if (args.num_fnames == 0) {
-			// Hardcode grep-dctrl <-> "-" mapping so that
-			// Debian packages can genuinely depend on it.
 			char * argv0 = fnbase(argv[0]);
 			if (strcmp(argv0, "grep-dctrl") == 0) {
 				fname = (struct ifile){ .mode = m_read,
 							.s = "-" };
+                        } else if (strcmp(argv0, "grep-status") == 0) {
+                                fname = (struct ifile){
+                                        .mode = m_read,
+                                        .s = "/var/lib/dpkg/status" };
+                        } else if (strcmp(argv0, "grep-available") == 0) {
+                                fname = (struct ifile){
+                                        .mode = m_read,
+                                        .s = "/var/lib/dpkg/available" };
+                        } else if (strcmp(argv0, "grep-aptavail") == 0) {
+                                fname = (struct ifile){
+                                        .mode = m_exec,
+                                        .s = "apt-cache dumpavail" };
+                        } else if (strcmp(argv0, "grep-debtags") == 0) {
+                                fname = (struct ifile){
+                                        .mode = m_exec,
+                                        .s = "debtags dumpavail" };
 			} else {
-				fname = find_ifile_by_exename(argv0, args.rcname);
+                                message(L_FATAL, 0,
+                                        _("executable name '%s' is not recognised"),
+                                        argv0);
+                                fail();
 			}
 		} else {
 			fname = args.fname[i];
@@ -889,13 +960,24 @@ int main (int argc, char * argv[])
 		para_init(&pp, &para);
 		while (1) {
 			para_parse_next(&para);
-			if (para_eof(&pp)) break;
-			if ((args.invert_match 
-			     || !does_para_satisfy(args.p, &para))
-			    && (!args.invert_match 
-				|| does_para_satisfy(args.p, &para))) {
-				continue;
-			}
+			if (para_eof(&pp)) {
+                                if (args.only_file_name && args.invert_match) {
+                                        printf("%s\n", fname.s);
+                                        found = true;
+                                }
+                                break;
+                        }
+                        bool sat = does_para_satisfy(args.p, &para);
+                        if (args.only_file_name) {
+                                if (!sat) continue;
+                                if (!args.invert_match) {
+                                        printf("%s\n", fname.s);
+                                        found = true;
+                                }
+                                break;
+                        }
+                        if (sat == args.invert_match) continue;
+
 			if (args.quiet) {
 				exit(0);
 			}
@@ -904,41 +986,9 @@ int main (int argc, char * argv[])
 				++count;
 				continue;
 			}
-			if (args.num_show_fields == 0) {
-				struct fsaf_read_rv r = get_whole_para(&para);
-				fwrite(r.b, 1, r.len, stdout);
-				putchar('\n');
-				putchar('\n');
-				continue;
-			}
-                        if (args.invert_show) {
-                                for (size_t j = 0;
-                                     j < fieldtrie_count() &&
-                                             j < para.nfields; 
-                                     j++) {
-                                        struct field_attr *fa = 
-                                                fieldtrie_get(j);
-                                        if (IS_SHOW_FIELD(fa->application_data)) {
-                                                continue;
-                                        }
-                                        show_field(&args, &para, fa);
-                                }
-                        } else {
-                                for (size_t j = 0;
-                                     j < args.num_show_fields; j++) {
-                                        size_t inx = args.show_fields[j];
-                                        struct field_attr *fa = 
-                                                fieldtrie_get(inx);
-                                        assert(IS_SHOW_FIELD
-                                               (fa->application_data));
-                                        show_field(&args, &para, fa);
-                                }
-                                if ((args.show_field_name &&
-                                     args.ensure_dctrl) ||
-                                    args.num_show_fields > 1) puts("");
-                        }
-		}
 
+                        print_para(&args, &para);
+                }
 		fsaf_close(fp);
 		close_ifile(fname, fd);
 	}
