@@ -1,5 +1,5 @@
 /*  dctrl-tools - Debian control file inspection tools
-    Copyright © 2004 Antti-Juhani Kaijanaho
+    Copyright © 2004, 2010 Antti-Juhani Kaijanaho
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -11,15 +11,15 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "msg.h"
 #include "para_bundle.h"
 
-void bundle_slurp(struct para_bundle * pb, struct ifile ifi)
+void bundle_slurp(struct para_bundle * pb, struct ifile ifi, _Bool reg)
 {
 	int fd = open_ifile(ifi);
 	if (fd == -1) {
@@ -39,7 +39,7 @@ void bundle_slurp(struct para_bundle * pb, struct ifile ifi)
 	sf->next = pb->files;
 	pb->files = sf;
 
-	para_parser_init(&sf->pp, f, false, false, false);
+	para_parser_init(&sf->pp, f, false, false, reg);
 	while (true) {
 		if (pb->num_paras == pb->max_num) {
 			size_t max_num = pb->max_num == 0 ? 256 :
